@@ -31,10 +31,12 @@ from ultima_scraper_db.databases.ultima_archive.schemas.templates.site import (
 from ultima_scraper_db.databases.ultima_archive.schemas.templates.site import (
     StoryModel as DBStoryModel,
 )
+from ultima_scraper_db.databases.ultima_archive.site_api import content_model_types
 
 api_types = ultima_scraper_api.api_types
 user_types = ultima_scraper_api.user_types
 content_types = ultima_scraper_api.content_types
+invalid_subdomains = ["us", "uk", "ca", "ca2", "de", "sg"]
 
 
 class DBContentExtractor:
@@ -192,12 +194,21 @@ class ApiExtractor:
             if "mimetype" in asset_metadata:
                 raw_media_type: str = asset_metadata["mimetype"].split("/")[0]
             author = self.item.get_author()
+            if not raw_media_type:
+                if ".stream" in asset_metadata["files"]["full"]["url"]:
+                    continue
+                else:
+                    print(
+                        f"Authed: {author.username} - {self.item.__class__.__name__} - {self.item.id} - {raw_media_type}"
+                    )
             media_type = author.get_api().MediaTypes().find_by_value(raw_media_type)
             if "createdAt" in asset_metadata:
                 if isinstance(asset_metadata["createdAt"], str):
                     media_created_at = datetime.fromisoformat(
                         asset_metadata["createdAt"]
                     )
+                elif asset_metadata["createdAt"] == None:
+                    media_created_at = self.item.created_at
                 else:
                     media_created_at = datetime.fromtimestamp(
                         asset_metadata["createdAt"]
@@ -224,12 +235,11 @@ class ApiExtractor:
             if authed.drm:
                 new_asset.drm = bool(authed.drm.has_drm(asset_metadata))
             new_asset.urls = []
-            matches = ["us", "uk", "ca", "ca2", "de"]
             for url in [main_url, preview_url].copy():
                 if url:
                     if url.hostname:
                         subdomain = url.hostname.split(".")[1]
-                        if any(subdomain in nm for nm in matches):
+                        if any(subdomain in nm for nm in invalid_subdomains):
                             subdomain = url.hostname.split(".")[1]
                             if "upload" in subdomain:
                                 continue
@@ -251,8 +261,9 @@ class ApiExtractor:
 
     def resolve_paid(self):
         if hasattr(self.item, "price"):
-            if all(media["canView"] for media in self.item.media):
-                return True
+            if self.item.price and self.item.price > 0:
+                if all(media["canView"] for media in self.item.media):
+                    return True
             return False
         else:
             return None
@@ -284,7 +295,7 @@ class ContentMetadata:
         self.is_from_queue: bool = False
         self.__raw__: Any | None = None
         self.__soft__: Any = None
-        self.__db_content__: DBStoryModel | DBPostModel | DBMessageModel | None = None
+        self.__db_content__: content_model_types | None = None
         self.__legacy__ = False
         self.content_manager = content_manager
 
@@ -348,6 +359,8 @@ class MediaMetadata:
         drm: bool = False,
     ) -> None:
         self.id = int(media_id) if media_id is not None else None
+        if content_metadata:
+            self.user_id = content_metadata.user_id
         self.media_type = media_type
         self.urls: list[str] = urls
         self.preview = preview
@@ -361,6 +374,7 @@ class MediaMetadata:
         self.__content_metadata__ = content_metadata
         self.__db_media__: DBMediaModel | None = None
         self.content_manager = content_manager
+        self.drm_download_paths: list[Path] = []
         assert self.id
         self.content_manager.media_manager.medias[self.id] = self
 
@@ -378,13 +392,12 @@ class MediaMetadata:
         self.drm = bool(author.get_authed().drm.has_drm(raw_media))
         main_url = self.url_picker(raw_media)
         preview_url = self.preview_url_picker(raw_media)
-        matches = ["us", "uk", "ca", "ca2", "de"]
         self.urls = []
         for url in [main_url, preview_url].copy():
             if url:
                 if url.hostname:
                     subdomain = url.hostname.split(".")[1]
-                    if any(subdomain in nm for nm in matches):
+                    if any(subdomain in nm for nm in invalid_subdomains):
                         subdomain = url.hostname.split(".")[1]
                         if "upload" in subdomain:
                             continue
@@ -494,7 +507,6 @@ class MetadataManager:
                 )
                 new_metadata_filepath.parent.mkdir(parents=True, exist_ok=True)
                 self.filesystem_manager.move(legacy_filepath, new_metadata_filepath)
-        await self.filesystem_manager.format_directories(self.subscription)
         await self.fix_archived_db()
         self.metadatas = self.find_metadatas()
 
@@ -516,9 +528,12 @@ class MetadataManager:
                 continue
             final_content_type = None
             archive = False
-            new_metadata_set: list[dict[str, Any]] | dict[
-                str, Any
-            ] = main_helper.import_json(metadata_filepath)
+            if metadata_filepath.stat().st_size == 0:
+                metadata_filepath.unlink()
+                continue
+            new_metadata_set: list[dict[str, Any]] | dict[str, Any] = (
+                main_helper.import_json(metadata_filepath)
+            )
             content_types = self.subscription.get_api().CategorizedContent().get_keys()
             final_stem = metadata_filepath.stem
             patterns = []
@@ -542,6 +557,8 @@ class MetadataManager:
                 if final_stem == "Archived" or "Archived" in metadata_filepath.parts:
                     archive = True
                     final_content_type = "Posts"
+                if "Posts" in final_stem:
+                    final_content_type = "Posts"
                 if not final_content_type:
                     # If we get an key error here, we need to move the json file to correct folder or we can resolve it by getting content_type by looking at the directory path
                     # directory_set = set()
@@ -567,7 +584,12 @@ class MetadataManager:
                                 for item in new_metadata_set["valid"]
                             ):
                                 final_content_type = temp_content_type
+                                if not final_content_type:
+                                    if metadata_filepath.parent.stem == "Metadata":
+                                        continue
                         else:
+                            if "content_type" not in new_metadata_set:
+                                continue
                             final_content_type = new_metadata_set["content_type"]
             assert (
                 final_content_type
@@ -579,7 +601,7 @@ class MetadataManager:
                 for temp_set in new_metadata_set:
                     content_json[temp_set["type"]] = {
                         "valid": temp_set["valid"],
-                        "invalid": temp_set["invalid"],
+                        "invalid": temp_set.get("invalid", []),
                     }
                 final_metadata_set["version"] = 1.8
                 final_metadata_set["content"] = content_json
@@ -772,9 +794,10 @@ class MetadataManager:
                     archived_database_path.name,
                 )
                 new_filepath.parent.mkdir(exist_ok=True)
-                archived_database_path.rename(new_filepath)
-                await filesystem_manager_user.remove_file(archived_database_path)
-                await filesystem_manager_user.add_file(new_filepath)
+                filesystem_manager_user.rename_path(
+                    archived_database_path, new_filepath
+                )
+                await filesystem_manager_user.refresh_files()
 
     def export(self, api_type: str, datas: list[dict[str, Any]]):
         if api_type == "Posts":
