@@ -1,17 +1,36 @@
 from __future__ import annotations
 
+import asyncio
 import copy
-from itertools import product
-from typing import Any, Optional
+from typing import Any
 
 import ultima_scraper_api
+from rich.progress import (
+    BarColumn,
+    Progress,
+    SpinnerColumn,
+    TextColumn,
+    TimeElapsedColumn,
+)
 from sqlalchemy import and_, or_, select
 from sqlalchemy.orm import joinedload
-from tqdm.asyncio import tqdm_asyncio
 from ultima_scraper_api.apis.onlyfans.classes.auth_model import OnlyFansAuthModel
 from ultima_scraper_api.apis.onlyfans.classes.user_model import (
-    create_user as OnlyFansUserModel,
+    UserModel as OnlyFansUserModel,
 )
+from ultima_scraper_api.helpers.main_helper import ProgressBar
+from ultima_scraper_collection.config import site_config_types
+from ultima_scraper_collection.managers.content_manager import (
+    ContentManager,
+    MediaManager,
+)
+from ultima_scraper_collection.managers.download_manager import DownloadManager
+from ultima_scraper_collection.managers.filesystem_manager import FilesystemManager
+from ultima_scraper_collection.managers.metadata_manager.metadata_manager import (
+    MediaMetadata,
+    MetadataManager,
+)
+from ultima_scraper_collection.managers.server_manager import ServerManager
 from ultima_scraper_db.databases.ultima_archive.schemas.templates.site import (
     FilePathModel as DBFilePathModel,
 )
@@ -27,19 +46,6 @@ from ultima_scraper_db.databases.ultima_archive.schemas.templates.site import (
 )
 from ultima_scraper_renamer.reformat import ReformatManager
 
-from ultima_scraper_collection.config import site_config_types
-from ultima_scraper_collection.managers.content_manager import (
-    ContentManager,
-    MediaManager,
-)
-from ultima_scraper_collection.managers.download_manager import DownloadManager
-from ultima_scraper_collection.managers.filesystem_manager import FilesystemManager
-from ultima_scraper_collection.managers.metadata_manager.metadata_manager import (
-    MediaMetadata,
-    MetadataManager,
-)
-from ultima_scraper_collection.managers.server_manager import ServerManager
-
 auth_types = ultima_scraper_api.auth_types
 user_types = ultima_scraper_api.user_types
 message_types = ultima_scraper_api.message_types
@@ -48,6 +54,7 @@ subscription_types = ultima_scraper_api.subscription_types
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
+    from dashboard import WorkerDashboardApp
     from ultima_scraper_collection.managers.datascraper_manager.datascrapers.fansly import (
         FanslyDataScraper,
     )
@@ -58,29 +65,6 @@ if TYPE_CHECKING:
     datascraper_types = OnlyFansDataScraper | FanslyDataScraper
 
 
-class download_session(tqdm_asyncio):
-    def start(
-        self,
-        unit: str = "B",
-        unit_scale: bool = True,
-        miniters: int = 1,
-        tsize: int = 0,
-    ):
-        self.unit = unit
-        self.unit_scale = unit_scale
-        self.miniters = miniters
-        self.total = 0
-        self.colour = "Green"
-        if tsize:
-            tsize = int(tsize)
-            self.total += tsize
-
-    def update_total_size(self, tsize: Optional[int]):
-        if tsize:
-            tsize = int(tsize)
-            self.total += tsize
-
-
 async def find_earliest_non_downloaded_message(
     user: user_types, datascraper: "datascraper_types"
 ):
@@ -88,47 +72,39 @@ async def find_earliest_non_downloaded_message(
     site_api = datascraper.server_manager.ultima_archive_db_api.get_site_api(
         authed.get_api().site_name
     )
-    temp_db_messages = await site_api.schema.session.scalars(
-        select(DBMessageModel)
-        .where(
-            or_(
-                and_(
-                    DBMessageModel.user_id == authed.id,
-                    DBMessageModel.receiver_id == user.id,
-                ),
-                and_(
-                    DBMessageModel.user_id == user.id,
-                    DBMessageModel.receiver_id == authed.id,
-                ),
-            )
-        )
-        .order_by(DBMessageModel.id.desc())
-        .options(
-            joinedload(DBMessageModel.media)
-            .joinedload(DBMediaModel.filepaths)
-            .joinedload(DBFilePathModel.message)
-        )
-    )
-
-    db_messages = temp_db_messages.unique().all()
     earliest_non_downloaded_message = None
-
+    found_media = False
+    db_performer = datascraper.db_performers[user.id]
+    db_content_manager = db_performer.content_manager
+    assert db_content_manager, "Content manager not found"
+    db_messages = db_content_manager.messages
     for db_message in db_messages:
-        all_media_downloaded = False
-        for media in db_message.media:
-            for filepath in media.filepaths:
-                if not filepath.message:
-                    continue
-                if filepath.downloaded:
-                    all_media_downloaded = True
-                    break
+        if (db_message.user_id == authed.id and db_message.receiver_id == user.id) or (
+            db_message.user_id == user.id and db_message.receiver_id == authed.id
+        ):
+            all_media_downloaded = False
+            if db_message.media:
+                found_media = True
+            else:
+                continue
+            for media in db_message.media:
+                for filepath in media.filepaths:
+                    if not filepath.message:
+                        continue
+                    if filepath.downloaded:
+                        all_media_downloaded = True
+                        break
 
-        if not all_media_downloaded:
-            if (
-                earliest_non_downloaded_message is None
-                or db_message.created_at < earliest_non_downloaded_message.created_at
-            ):
-                earliest_non_downloaded_message = db_message
+            if not all_media_downloaded:
+                if (
+                    earliest_non_downloaded_message is None
+                    or db_message.created_at
+                    < earliest_non_downloaded_message.created_at
+                ):
+                    earliest_non_downloaded_message = db_message
+    if found_media:
+        earliest_non_downloaded_message = db_messages[0]
+    await site_api.schema.session.commit()
     return earliest_non_downloaded_message
 
 
@@ -140,10 +116,13 @@ class StreamlinedDatascraper:
         self.filesystem_manager = FilesystemManager()
         self.media_types = self.datascraper.api.MediaTypes()
         self.user_list: set[user_types] = set()
+        self.db_performers: dict[int, UserModel] = {}
         self.metadata_manager_users: dict[int, MetadataManager] = {}
         self.server_manager: ServerManager = server_manager
         self.content_managers: dict[int, ContentManager] = {}
         self.media_managers: dict[int, MediaManager] = {}
+        self.dashboard: "WorkerDashboardApp | None" = None
+        self.worker_id: int = 0
 
     def find_metadata_manager(self, user_id: int):
         return self.metadata_manager_users[user_id]
@@ -173,7 +152,7 @@ class StreamlinedDatascraper:
         available_jobs = site_config.jobs.scrape
         option_manager = self.datascraper.option_manager
         performer_options = option_manager.performer_options
-        assert option_manager.subscription_options
+        assert option_manager.subscription_options, "Subscription options not found"
         valid_user_list: set[user_types] = set(
             option_manager.subscription_options.final_choices
         )
@@ -197,7 +176,8 @@ class StreamlinedDatascraper:
             [
                 user.scrape_whitelist.append("Messages")
                 for user in chat_users
-                if not user.is_subscribed() or not scraping_subscriptions
+                if not user.is_user_subscribed_to_performer()
+                or not scraping_subscriptions
             ]
             [valid_user_list.add(x) for x in chat_users]
 
@@ -220,7 +200,7 @@ class StreamlinedDatascraper:
                                 performer.scrape_whitelist.clear()
                                 valid_user_list.add(performer)
         from ultima_scraper_api.apis.fansly.classes.user_model import (
-            create_user as FYUserModel,
+            UserModel as FYUserModel,
         )
 
         for user in valid_user_list:
@@ -247,10 +227,14 @@ class StreamlinedDatascraper:
             return
         authed: auth_types = user.get_authed()
         site_config = self.datascraper.site_config
-        if isinstance(authed, OnlyFansAuthModel) and user.is_authed_user():
+        if (
+            isinstance(authed, OnlyFansAuthModel)
+            and user.is_authed_user()
+            and user.is_performer()
+        ):
             vault = await authed.get_vault_lists()
             vault_item = vault.resolve(name=content_type)
-            assert vault_item
+            assert vault_item, f"Vault item {content_type} not found"
             vault_item_medias = await vault_item.get_medias()
             media_metadatas: list[MediaMetadata] = []
             for vault_item_media in vault_item_medias:
@@ -292,7 +276,7 @@ class StreamlinedDatascraper:
     async def paid_content_scraper(self, authed: auth_types):
         paid_contents = await authed.get_paid_content()
         datascraper = self.datascraper
-        assert datascraper
+        assert datascraper, "Datascraper not found"
         unique_suppliers: set[user_types] = set()
         for paid_content in paid_contents:
             supplier = paid_content.get_author()
@@ -344,6 +328,10 @@ class StreamlinedDatascraper:
                     pass
                 case "Posts":
                     temp_master_set = await self.datascraper.get_all_posts(user)
+                    # temp_master_set = [
+                    #     x for x in temp_master_set if x.id in [1264067231, 306722777]
+                    # ]
+                    pass
                 case "Messages":
                     db_message = await find_earliest_non_downloaded_message(
                         user, self.datascraper
@@ -351,7 +339,13 @@ class StreamlinedDatascraper:
                     cutoff_id = db_message.id if db_message else None
                     temp_master_set = await user.get_messages(cutoff_id=cutoff_id)
                 case "Chats":
-                    pass
+                    if isinstance(authed, OnlyFansAuthModel):
+                        if user.is_authed_user():
+                            chats = await authed.get_chats()
+                            temp_master_set = []
+                            for chat in chats:
+                                messages = await chat.get_messages()
+                                temp_master_set.extend(messages)
                 case "Highlights":
                     pass
                 case "MassMessages":
@@ -376,7 +370,7 @@ class StreamlinedDatascraper:
                 case _:
                     raise Exception(f"{content_type} is an invalid choice")
         # Adding paid content and removing duplicates by id
-        if isinstance(user, ultima_scraper_api.onlyfans_classes.user_model.create_user):
+        if isinstance(user, ultima_scraper_api.onlyfans_classes.user_model.UserModel):
             for paid_content in await user.get_paid_contents(content_type):
                 temp_master_set.append(paid_content)
             pass
@@ -397,34 +391,44 @@ class StreamlinedDatascraper:
     ):
         if not master_set:
             return False
-        # if  api_type != "Posts":
-        #     return
-        authed = subscription.get_authed()
         unrefined_set = []
-        with authed.get_pool() as pool:
-            print(f"Processing Scraped {api_type}")
-            tasks = pool.starmap(
-                self.datascraper.media_scraper,
-                product(
-                    master_set,
-                    [subscription],
-                    [api_type],
-                ),
+        tasks = [
+            asyncio.create_task(
+                self.datascraper.media_scraper(x, subscription, api_type)  # type:ignore
             )
-            settings = {"colour": "MAGENTA"}
-            unrefined_set: list[dict[str, Any]] = await tqdm_asyncio.gather(
-                *tasks, **settings
+            for x in master_set
+        ]
+        dashboard = self.dashboard
+        assert dashboard
+        total = len(tasks)
+        completed = 0
+        worker = dashboard.get_worker(self.worker_id)
+        for task in asyncio.as_completed(tasks):
+            result = await task
+            unrefined_set.append(result)
+            completed += 1
+            # Update dashboard scraping progress
+            dashboard.set_scrape_task(
+                self.worker_id,
+                (completed / total) * 100,
+                f"{subscription.username} | {api_type} ({completed}/{total})",
             )
-            new_metadata = metadata_manager.merge_content_and_directories(unrefined_set)
-            final_content, _final_directories = new_metadata
-            if new_metadata:
-                new_metadata_content = final_content
-                content_manager = self.resolve_content_manager(subscription)
-                content_manager.set_content(api_type, new_metadata_content)
-                if new_metadata_content:
-                    pass
-            else:
-                print(f"No {api_type} found.")
+            await asyncio.sleep(0)
+
+        new_metadata = metadata_manager.merge_content_and_directories(unrefined_set)
+        final_content, _final_directories = new_metadata
+        if new_metadata:
+            new_metadata_content = final_content
+            content_manager = self.resolve_content_manager(subscription)
+            content_manager.set_content(api_type, new_metadata_content)
+            if new_metadata_content:
+                pass
+        else:
+            print(f"No {api_type} found.")
+        # Mark as done in dashboard
+        dashboard.set_scrape_task(
+            self.worker_id, 100, f"{subscription.username} | {api_type} (Done)"
+        )
         return True
 
     # Downloads scraped content
@@ -446,6 +450,14 @@ class StreamlinedDatascraper:
         final_download_set: set[MediaMetadata] = set()
         for db_media in db_medias.values():
             content_info = None
+            if api_type == "Uncategorized":
+                media_metadata = content_manager.media_manager.medias.get(db_media.id)
+            else:
+                media_metadata = content_manager.find_media(
+                    category=api_type, media_id=db_media.id
+                )
+            if not media_metadata:
+                continue
             if api_type == "Uncategorized":
                 await db_media.awaitable_attrs.content_media_assos
                 if db_media.content_media_assos:
@@ -453,40 +465,43 @@ class StreamlinedDatascraper:
                 if len(db_media.filepaths) > 1:
                     continue
             else:
-                db_content = await db_media.find_content(api_type)
+                content_metadata = media_metadata.get_content_metadata()
+                if not content_metadata:
+                    continue
+                db_content = await db_media.find_content(
+                    api_type, content_metadata.content_id
+                )
                 if not db_content:
                     continue
                 content_info = (db_content.id, api_type)
             db_filepath = db_media.find_filepath(content_info)
             if db_filepath:
-                if api_type == "Uncategorized":
-                    media_metadata = content_manager.media_manager.medias.get(
-                        db_media.id
-                    )
-                else:
-                    media_metadata = content_manager.find_media(
-                        category=api_type, media_id=db_media.id
-                    )
-                if media_metadata:
+                if media_metadata and media_metadata.urls:
                     media_metadata.__db_media__ = db_media
                     final_download_set.add(media_metadata)
         total_media_count = len(final_download_set)
-        download_media_count = 0
+        non_downloaded = [
+            x for x in final_download_set if not x.get_filepath().exists()
+        ]
+        if non_downloaded:
+            pass
+        download_media_count = len(non_downloaded)
         directory = performer_directory_manager.user.download_directory
-        if final_download_set:
-            string = "Processing Download:\n"
-            string += f"Name: {performer.username} | Type: {api_type} | Downloading: {download_media_count} | Total: {total_media_count} | Directory: {directory}\n"
-            print(string)
         download_manager = DownloadManager(
             performer.get_authed(),
             filesystem_manager,
             final_download_set,
             global_settings.tools.reformatter.active,
+            self.dashboard,
+            self.datascraper.worker_id,
+        )
+        status = await download_manager.bulk_download(
+            performer,
+            api_type,
+            download_media_count,
+            total_media_count,
         )
-        await download_manager.bulk_download()
         await site_db_api.schema.session.commit()
-        if final_download_set:
-            pass
         if current_job:
             current_job.done = True
 
@@ -571,19 +586,25 @@ class StreamlinedDatascraper:
                     #     return None
                     return None
                 else:
-                    performer = paid_contents[0].get_author()
+                    performer = [
+                        x.get_author()
+                        for x in paid_contents
+                        if x.get_author().id == db_performer.id
+                    ][0]
                     temp_performer = await authed.get_user(performer.id, refresh=True)
                     if not temp_performer:
                         performer.is_deleted = True
             else:
                 performer = subscriptions[0].user
-                if not performer.is_subscribed():
+                if not performer.is_user_subscribed_to_performer():
                     paid_contents = await authed.get_paid_content(
                         performer_id=db_performer.id
                     )
         if isinstance(
-            performer, ultima_scraper_api.onlyfans_classes.user_model.create_user
+            performer, ultima_scraper_api.onlyfans_classes.user_model.UserModel
         ):
             if performer.is_blocked:
                 await performer.unblock()
+        performer.add_aliases([x.username for x in db_performer.aliases])
+        performer.username = performer.get_usernames()[0]
         return performer
